#...........................
# Homework 4 related explorations - Mike ####
#...........................

# Reminder! Some of this code, written by Mike, is very close to exact answers to the homework. 
# Please consider the honor code and ensure you are making your own efforts when you get to the homework assignments.

library(tidyverse)
library(broom)
library(forcats)

load("D:/User/Dropbox (Personal)/Education/Classes/18Fall_EPID799C_RforEpi/data/R for epi 2018 data pack/births_sm.rdata")
births_sm = as.tibble(births_sm)

# Quick factor checks
levels(births_sm$preterm_f)
births_sm$preterm_f = fct_rev(births_sm$preterm_f)
levels(births_sm$pnc5_f)

levels(births_sm$raceeth_f)
births_sm$raceeth_f = fct_rev(births_sm$raceeth_f)
births_sm$raceeth_f = relevel(births_sm$raceeth_f, 
                              ref = "White non-Hispanic")


# Q1
crude_risks = births_sm %>% 
  group_by(pnc5_f, preterm_f) %>% 
  summarize(n=n()) %>% mutate(pct = n / sum(n)) %>%
  filter(preterm_f == "preterm", !is.na(pnc5_f))
r0 = crude_risks$pct[crude_risks$pnc5_f == "No Early PNC"]
r1 = crude_risks$pct[crude_risks$pnc5_f == "Early PNC"]
r1 - r0

as.data.frame(prop.table(table(pnc = births_sm$pnc5_f, preterm = births_sm$preterm_f), 
           margin = 1)) %>% filter(preterm == "preterm")

#Q2
# lag

births_sm %>% 
  group_by(raceeth_f, pnc5_f, preterm_f) %>% 
  summarize(n=n()) %>% 
  mutate(pct = n / sum(n)) %>%
  filter(!is.na(pnc5_f), preterm_f == "preterm") %>%
  group_by(raceeth_f) %>% mutate(rd = pct-lag(pct)) %>%
  filter(pnc5_f == "Early PNC")
1:10; lag(1:10)

rd_df = births_sm %>% 
  group_by(raceeth_f, pnc5_f, preterm_f) %>% 
  summarize(n=n()) %>% 
  mutate(pct = n / sum(n)) %>%
  filter(!is.na(pnc5_f), preterm_f == "preterm") %>%
  select(-preterm_f, -n) %>% 
  spread(pnc5_f, pct) %>%
  mutate(RD = `No Early PNC` - `Early PNC`)

as.data.frame(table(births_sm$preterm_f, births_sm$pnc5_f, births_sm$raceeth_f))

rd_tbl = as.data.frame(prop.table(table(preterm = births_sm$preterm_f, 
                                        pnc5 = births_sm$pnc5_f, 
                                        raceeth = births_sm$raceeth_f), 
                                  margin = c(2,3)))

# Q3
births_risk_df = births_sm %>% 
  group_by(raceeth_f, pnc5_f, preterm_f) %>% # careful with groups
  summarize(n=n()) %>% 
  mutate(pct = n / sum(n)) %>%
  filter(!is.na(pnc5_f), preterm_f == "preterm")
  
births_risk_df %>%
  ggplot(aes(pnc5_f, pct, color=raceeth_f, group=raceeth_f))+
  geom_line(size = 1)+geom_point(aes(size=n))+
  geom_hline(yintercept = 0.125, lty=2)+
  annotate(label="EMM note!", x=0.8, y=0.14, geom="text")+
  scale_y_continuous(limits=c(0,NA))+
  labs(title="Quick look at crude EMM of RD by race-ethnicity", 
       subtitle="Note possible EMM (angle of lines)... and underlying disparities!",
       x="Prenatal Care", y="Prevalence of Preterm Birth", 
       caption="n = 62000 or so, data from NC SCHS 2012.") + 
  theme_bw()


# Q4

head(births_sm)
births_sm$mage_sq = births_sm$mage ^ 2
m1 = glm(formula = preterm_f ~ pnc5_f, family = binomial("identity"), data = births_sm)
m2 = glm(formula = preterm_f ~ pnc5_f + smoker_f , family = binomial("identity"), data = births_sm)
m3 = glm(formula = preterm_f ~ pnc5_f + smoker_f + mage, family = binomial("identity"), data = births_sm)
m4 = glm(formula = preterm_f ~ pnc5_f + smoker_f + poly(mage, 2, raw = T), family = binomial("identity"), data = births_sm)

# method 0: no dplyr or purrr. yuck.
coef(m1); confint(m1)
c(coef(m1)[2], confint(m1)[2,])
m1_summary = summary(m1)
m1_summary$coefficients

c(coef(m1)[2], confint(m1)[2,], 
  std.error = m1_summary$coefficients[2,2])

m1_df = c(coef(m1)[2], confint(m1)[2,], std.error = m1_summary$coefficients[2,2])
m2_df = c(coef(m2)[2], confint(m2)[2,], std.error = summary(m2)$coefficients[2,2])
m3_df = c(coef(m3)[2], confint(m3)[2,], std.error = summary(m3)$coefficients[2,2])
m4_df = c(coef(m4)[2], confint(m4)[2,], std.error = summary(m4)$coefficients[2,2])

results_df = data.frame(rbind(m1_df, m2_df, m3_df, m4_df))
results_df$model_name = row.names(results_df)

# method 1: dplyr & broom
m1_df = bind_cols(tidy(m1), confint_tidy(m1)) %>% filter(term == "pnc5_fEarly PNC") %>% mutate(model_name="M1: Crude")
m2_df = bind_cols(tidy(m2), confint_tidy(m2)) %>% filter(term == "pnc5_fEarly PNC") %>% mutate(model_name="M2: ... + smoking")
m3_df = bind_cols(tidy(m3), confint_tidy(m3)) %>% filter(term == "pnc5_fEarly PNC") %>% mutate(model_name="M3: ... + mage")
m4_df = bind_cols(tidy(m4), confint_tidy(m4)) %>% filter(term == "pnc5_fEarly PNC") %>% mutate(model_name="M4: ... + mage^2")
#....
results_df = bind_rows(m1_df, m2_df, m3_df, m4_df)
results_df

# method 2: dplyr & purrr
model_results = function(m){bind_cols(tidy(m), confint_tidy(m)) %>% filter(term == "pnc5_fEarly PNC")}
model_results(m1)

model_df = tibble(models = list(m1, m2, m3, m4), 
                  model_name = paste0("M", 1:4), 
                  results = map(models, model_results), 
                  estimate = map_dbl(results, "estimate")) # accessor in a list
model_df
results_df = bind_rows(model_df$results) %>% 
  mutate(model_name = model_df$model_name)


# method 3.... fancier!
vars = c("preterm_f", "pnc5_f", "smoker_f", "mage", "mage_sq") # predictors
model_maker = function(v) { glm(preterm_f ~ ., family=binomial("identity"), data=births_sm[,v]) }
model_df = tibble(covariates = map(2:length(vars), ~ vars[1:.]), models = map(covariates, model_maker)) 
#....

# plotting
ggplot(results_df, aes(model_name, color=std.error)) + 
  geom_point(aes(y=estimate)) + 
  geom_linerange(aes(ymin = conf.low, ymax=conf.high))
